@@ -5,6 +5,7 @@ use log::*;
 use serde::{Deserialize, Serialize};
 
 use crate::manager::*;
+use crate::wg_dev::map_to_ipv6;
 
 #[derive(Serialize, Deserialize, Clone, Debug, Default, PartialEq)]
 pub struct PublicKeyWithTime {
@@ -152,6 +153,7 @@ impl StaticConfiguration {
             lines.push("[Peer]".to_string());
             lines.push(format!("PublicKey = {}", &peer.public_key.key));
             lines.push(format!("AllowedIPs = {}/32", peer.wg_ip));
+            lines.push(format!("AllowedIPs = {}/128", map_to_ipv6(&peer.wg_ip)));
             let ips = manager.get_ips_for_peer(peer.wg_ip);
             for ip in ips {
                 lines.push(format!("AllowedIPs = {}/32", ip));
@@ -168,11 +170,25 @@ impl StaticConfiguration {
                 debug!(target: "configuration", "peer {} uses visible (NAT) endpoint {}", peer.wg_ip, endpoint);
                 debug!(target: &peer.wg_ip.to_string(), "use visible (NAT) endpoint {}", endpoint);
                 lines.push(format!("EndPoint = {}", endpoint));
-                //                lines.push("PersistentKeepalive = 5".to_string());
+                // lines.push("PersistentKeepalive = 5".to_string());
             }
             lines.push("".to_string());
         }
 
+        for node in manager.known_nodes.values() {
+            if let Some(public_key) = node.public_key.as_ref() {
+                lines.push("[Peer]".to_string());
+                lines.push(format!("PublicKey = {}", &public_key.key));
+                lines.push(format!("AllowedIPs = {}/128", map_to_ipv6(&node.wg_ip)));
+                if let Some(endpoint) = node.visible_endpoint {
+                    debug!(target: "configuration", "node {} uses visible (NAT) endpoint {}", node.wg_ip, endpoint);
+                    debug!(target: &node.wg_ip.to_string(), "use visible (NAT) endpoint {}", endpoint);
+                    lines.push(format!("EndPoint = {}", endpoint));
+                    // lines.push("PersistentKeepalive = 5".to_string());
+                }
+                lines.push("".to_string());
+            }
+        }
         lines.join("\n")
     }
     pub fn my_admin_port(&self) -> u16 {
