@@ -32,6 +32,7 @@ use serde::{Deserialize, Serialize};
 use crate::configuration::*;
 use crate::crypt_udp::*;
 use crate::event::Event;
+use crate::wg_dev::map_to_ipv6;
 
 #[derive(Debug)]
 pub enum RouteChange {
@@ -87,15 +88,17 @@ pub struct DynamicPeer {
 #[derive(Debug)]
 pub struct Node {
     is_static_peer: Option<bool>,
-    wg_ip: Ipv4Addr,
+    pub wg_ip: Ipv4Addr,
     admin_port: u16,
     hop_cnt: usize,
     gateway: Option<Ipv4Addr>,
-    public_key: Option<PublicKeyWithTime>,
+    pub public_key: Option<PublicKeyWithTime>,
     known_in_s: usize,
     local_ip_list: Option<Vec<IpAddr>>,
     local_admin_port: Option<u16>,
     send_count: usize,
+    can_send_to_visible_endpoint: bool,
+    pub visible_endpoint: Option<SocketAddr>,
 }
 impl Node {
     fn from(ri: &RouteInfo) -> Self {
@@ -110,6 +113,8 @@ impl Node {
             local_ip_list: None,
             local_admin_port: None,
             send_count: 0,
+            can_send_to_visible_endpoint: false,
+            visible_endpoint: None,
         }
     }
     fn process_every_second(&mut self, static_config: &StaticConfiguration) -> Vec<Event> {
@@ -135,15 +140,25 @@ impl Node {
             return events;
         }
 
-        if self.local_ip_list.is_none() {
+        if self.local_ip_list.is_none()
+            || self.public_key.is_none()
+            || self.visible_endpoint.is_none()
+        {
             if self.known_in_s % 60 == 0 || self.known_in_s < 5 {
                 // Send request for local contact
                 let destination = SocketAddrV4::new(self.wg_ip, self.admin_port);
                 events.push(Event::SendLocalContactRequest { to: destination });
             }
+        } else if let Some(admin_port) = self.local_admin_port.as_ref() {
+            // All ok. so constantly send advertisement to the Ipv6 address
+            events.push(Event::SendAdvertisement {
+                addressed_to: AddressedTo::WireguardV6Address,
+                to: SocketAddr::new(IpAddr::V6(map_to_ipv6(&self.wg_ip)), *admin_port),
+                wg_ip: self.wg_ip,
+            });
         }
 
-        if self.send_count < 10 {
+        if self.send_count < 100 {
             if let Some(ip_list) = self.local_ip_list.as_ref() {
                 if let Some(admin_port) = self.local_admin_port.as_ref() {
                     self.send_count += 1;
@@ -163,11 +178,20 @@ impl Node {
             }
         }
 
+        let can_send = self.public_key.is_some() && self.visible_endpoint.is_some();
+
+        if can_send && !self.can_send_to_visible_endpoint {
+            self.can_send_to_visible_endpoint = true;
+            events.push(Event::UpdateWireguardConfiguration);
+        }
+
         events
     }
     fn process_local_contact(&mut self, local: LocalContactPacket) {
         self.local_ip_list = Some(local.local_ip_list);
         self.local_admin_port = Some(local.local_admin_port);
+        self.visible_endpoint = local.my_visible_wg_endpoint;
+        self.public_key = Some(local.public_key);
     }
     fn ok_to_delete_without_route(&self) -> bool {
         self.known_in_s > 10
@@ -181,7 +205,7 @@ pub struct NetworkManager {
     peer_route_db: HashMap<Ipv4Addr, PeerRouteDB>,
 
     pending_route_changes: Vec<RouteChange>,
-    known_nodes: HashMap<Ipv4Addr, Node>,
+    pub known_nodes: HashMap<Ipv4Addr, Node>,
     peer: HashMap<Ipv4Addr, DynamicPeer>,
     fifo_dead: Vec<Ipv4Addr>,
     fifo_ping: Vec<Ipv4Addr>,
@@ -229,6 +253,7 @@ impl NetworkManager {
     }
     pub fn analyze_advertisement(
         &mut self,
+        static_config: &StaticConfiguration,
         advertisement: AdvertisementPacket,
         src_addr: SocketAddr,
     ) -> Vec<Event> {
@@ -242,6 +267,7 @@ impl NetworkManager {
 
         let mut local_reachable_admin_endpoint = None;
         let mut local_reachable_wg_endpoint = None;
+        let mut dp_visible_wg_endpoint = None;
 
         use AddressedTo::*;
         match &advertisement.addressed_to {
@@ -251,9 +277,28 @@ impl NetworkManager {
                     Some(SocketAddr::new(src_addr.ip(), advertisement.local_wg_port));
                 local_reachable_admin_endpoint = Some(src_addr);
             }
+            WireguardV6Address | ReplyFromWireguardV6Address => {
+                dp_visible_wg_endpoint = advertisement.my_visible_wg_endpoint;
+            }
             WireguardAddress | ReplyFromWireguardAddress => {
                 if advertisement.your_visible_wg_endpoint.is_some() {
-                    self.my_visible_wg_endpoint = advertisement.your_visible_wg_endpoint;
+                    let mut is_local = false;
+
+                    for ip in static_config.ip_list.iter() {
+                        if *ip
+                            == advertisement
+                                .your_visible_wg_endpoint
+                                .as_ref()
+                                .unwrap()
+                                .ip()
+                        {
+                            is_local = true;
+                        }
+                    }
+
+                    if !is_local {
+                        self.my_visible_wg_endpoint = advertisement.your_visible_wg_endpoint;
+                    }
                 }
             }
         }
@@ -266,7 +311,7 @@ impl NetworkManager {
             name: advertisement.name.to_string(),
             local_reachable_admin_endpoint,
             local_reachable_wg_endpoint,
-            dp_visible_wg_endpoint: None,
+            dp_visible_wg_endpoint,
             admin_port: src_addr.port(),
             lastseen,
         };
